# frozen_string_literal: true

require 'forwardable'
require 'set'

module Zn
  # Holds repo, associators and performs aggregate operations
  class Dataset
    extend Forwardable
    attr_reader :repo, :associators

    def initialize(repo)
      @repo = repo
      @associators = {}
    end

    def add_associator(name, associator)
      @associators[name] = associator
    end

    def keys
      keys = Set.new

      repo.all.each do |entity|
        keys = keys.merge entity.keys
      end

      keys.to_a
    end

    def associations(entity)
      associators.each_with_object({}) do |(name, associator), hash|
        hash[name] = associator.associate(entity)
      end
    end

    def associate(entities)
      return enum_for(:associate, entities) unless block_given?

      entities.each do |entity|
        yield entity, associations(entity)
      end
    end

    delegate search: :@repo
  end
end
