# frozen_string_literal: true

require 'multi_json'

module Zn
  module Repos
    # JsonRepo abstracts loading of external json data. For more demanding
    # datasets, repos of other nature can be created later as needed. Examples
    # of this include loading data asynchronously from json files, loading data
    # from a database, elasticsearch, etc.
    class JsonRepo
      def initialize(params)
        dir = params.fetch('dir', '.')

        Dir.chdir(dir) do
          @io = File.new(params.fetch('file'))
        end
      end

      def all
        @all ||= MultiJson.load(@io)
      end

      def search(params)
        return enum_for(:search, params) unless block_given?

        key = params.fetch(:key)
        value = params.fetch(:value)
        exact = params.fetch(:exact)

        if exact || value.empty?
          search_exact(key, value) { |result| yield result }
        else
          search_for_inclusion(key, value) { |result| yield result }
        end
      end

      private

      def search_exact(key, value)
        return enum_for(:search_exact, key, value) unless block_given?

        all.each do |element|
          yield element if element.fetch(key, '').to_s == value.to_s
        end
      end

      def search_for_inclusion(key, value)
        return enum_for(:search_for_inclusion, key, value) unless block_given?

        all.each do |element|
          yield element if element.fetch(key, '').to_s.include?(value.to_s)
        end
      end
    end
  end
end
