# frozen_string_literal: true

module Zn
  # Provides a higher level API for search and relational logic
  class SearchAssembly
    attr_reader :datasets

    def initialize
      @datasets = {}
    end

    def add_dataset(name, dataset)
      @datasets[name] = dataset
    end

    def dataset_names
      datasets.keys
    end

    def dataset_keys(options)
      dataset(options).keys
    end

    def associate(entities, options)
      return enum_for(:associate, entities, options) unless block_given?

      dataset(options).associate(entities) do |entity, associations|
        yield entity.merge(associations)
      end
    end

    def search(options)
      return enum_for(:search, options) unless block_given?

      dataset(options).search(options) do |result|
        yield result
      end
    end

    private

    def dataset(options)
      dataset_name = options.fetch(:dataset)
      datasets.fetch(dataset_name)
    end
  end
end
